(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("TerritoryService", ["$q", "Endpoints", TerritoryService])
    ;

    function TerritoryService($q, endpoints) {
        var service = this;
        var territoryHierarchy = {};
        var userTerritoriesMap = {};

        service.isTM2 = null;

        service.getAccountsUnderUserTerritory = function(userId) {
            var deferred = $q.defer();
            var subQueries = [{
                name: "Territory",
                promise: getEnvironmentDependentTerritories()
            }, {
                name: "UserTerritory",
                promise: getEnvironmentDependentUserTerritories(userId)
            }];

            $q.all(_.map(subQueries, "promise")).then(function(subQueriesResp) {
                var allTerritories = subQueriesResp[0].Territory || subQueriesResp[0].Territory2;
                var userTerritories = subQueriesResp[1].UserTerritory || subQueriesResp[1].UserTerritory2Association;

                userTerritoriesMap = getQualifiedUserTerritories(allTerritories, userTerritories);

                endpoints.fetchAccountByTerritoryIds(_.map(userTerritoriesMap, "Id")).then(function(accountResp) {
                    accountResp = accountResp.map(function(acct) {
                        acct.territories = [];
                        acct._territoryIds.forEach(function(tId) {
                            acct.territories.push(userTerritoriesMap[tId]);
                        });
                        delete acct._territoryIds;
                        return acct;
                    });
                    deferred.resolve(accountResp);
                }, deferred.reject);
            }, deferred.reject);

            return deferred.promise;
        };

        service.getTerritoriesMapForCurrentUser = function() {
            return userTerritoriesMap;
        };

        service.getUserTerritories = function(userId) {
            var deferred = $q.defer();
            endpoints.fetchTerritoryByUser(userId).then(deferred.resolve, deferred.reject);
            return deferred.promise;
        };

        service.getUserTerritories2 = function(userId) {
            var deferred = $q.defer();
            endpoints.fetchTerritory2ByUser(userId).then(deferred.resolve, deferred.reject);
            return deferred.promise;
        };

        function getEnvironmentDependentTerritories() {
            if (service.isTM2) {

                return getAllTerritories2();
            }

            return getAllTerritories();
        }

        function getEnvironmentDependentUserTerritories(userId) {
            if (service.isTM2) {

                return service.getUserTerritories2(userId);
            }

            return service.getUserTerritories(userId);
        }

        function getAllTerritories() {
            var deferred = $q.defer();
            endpoints.fetchAllTerritories().then(deferred.resolve, deferred.reject);
            return deferred.promise;
        }

        function getAllTerritories2() {
            var deferred = $q.defer();
            endpoints.fetchAllTerritories2().then(deferred.resolve, deferred.reject);
            return deferred.promise;
        }

        function getQualifiedUserTerritories(allTerritories, userTerritories) {
            var territoryMap = {};

            // Initial pass to group territories by their parent
            allTerritories.forEach(function(t) {
                var parentId = (t.ParentTerritoryId || t.ParentTerritory2Id) || "root";
                if(!territoryHierarchy[parentId]) {
                    territoryHierarchy[parentId] = [];
                }
                territoryHierarchy[parentId].push(t);
                territoryMap[t.Id] = t;
            });

            var greedyChildIds = [];
            userTerritories.forEach(function(ut) {
                var childIds = findChildTerritories(ut.TerritoryId || ut.Territory2Id);
                greedyChildIds = greedyChildIds.concat(childIds);
            });

            var qualifiedUserTerritories = {};
            greedyChildIds.forEach(function(childId) {
                qualifiedUserTerritories[childId] = territoryMap[childId];
            });

            return qualifiedUserTerritories;
        }

        function findChildTerritories(baseTerritoryId) {
            if(!!territoryHierarchy[baseTerritoryId]) {
                var childIds = [baseTerritoryId];
                territoryHierarchy[baseTerritoryId].forEach(function(t) {
                    childIds = childIds.concat(findChildTerritories(t.Id));
                });
                return childIds;
            } else {
                return [baseTerritoryId];
            }
        }

        return service;
    }
}());
